/*
 * Authors 
 * - Rob Thomson <rob@marotori.com>

 * This tracker code was built to replace the analog tracker created by IBCrazy on
 * http://www.rcgroups.com/forums/showthread.php?t=1337608
 * 
 * The arduino is a 90% drop in replacement for the board.   The only difference.
 * with this system, simply use a propoer 360 degree servo.  Do not do the actual
 * mod required for the analog system.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#include <Servo.h>
#include <WProgram.h>

const int inputLeft = A0;     //left  rssi input
const int inputRight = A1;    //right rssi input
const int inputMidFront = A3;  //front rssi input
const int inputMidTop = A2;    //top rssi input
const int outputServo = 10;    //pin of servo used for pan motion
const int diversityServo = 11; //pin of 'servo' based fpv video switcher
const int servocenter = 90;  // absolute center. a 'home made' 360 will no doubt need a suitable value to be found.
int speedFast = 15;  //speed fast
int speedMedium = 10; //speed medium
int speedSlow = 5;  //speed slow
int speedVSlow = 2;  //speed very slow

/* 
these offsets can be used to find center the rssi as most modules are not exact to each other!
and easy way to find the exact values is to print the rssiLeft & rssiRight values - when the power
to the rf modules is turned off - well. that worked for me!
*/
int inputLeftOffset = 0;  
int inputRightOffset = 17; 

/*
  probably no need to edit below here.
*/

Servo servo;  // create servo object to control a servo
Servo diversity;  // create servo object to control a servo

void setup()
{ 
  Serial.begin(9600);
  servo.attach(outputServo);  
  diversity.attach(diversityServo);   
}

void loop() {
  
     int rssiLeft = analogRead(inputLeft) - inputLeftOffset ;
     int rssiRight = analogRead(inputRight) - inputRightOffset ;
     int rssiMidFront = analogRead(inputMidFront);
     int rssiMidTop = analogRead(inputMidTop);
     int rssiDiff = rssiLeft > rssiRight ? rssiLeft - rssiRight : rssiRight - rssiLeft;


    /* 
    ----------------------------------------------------------------------------- 
    DIVERTISTY SWITCHING LOGIC
    Only use if using a video switcher to change source feed depending on which
    antenna is best.
    
    note. this logic is using a 3 channel video switcher.  in my setup, the front
    antenna is actually a diversity antenna already - so I only have three video
    feeds.
    
    You may want to adjust logic to suit your setup, or just comment out if you
    do not use a video switcher.
    ----------------------------------------------------------------------------- 
    */
        if(rssiLeft > rssiRight && rssiLeft > rssiMidTop && rssiLeft > rssiMidFront){
               diversity.write(10); 
               
        } else if(rssiRight > rssiLeft && rssiRight > rssiMidTop && rssiRight > rssiMidFront){
               diversity.write(170); 
               
        } else {
               diversity.write(90); 
               
        }  
  
      /* 
    ----------------------------------------------------------------------------- 
    TRACKING LOGIC
    ----------------------------------------------------------------------------- 
    */
          if(rssiDiff  < 5){
            /* 
             if the rssi difference is not to high - dont move at all.
             this helps avoid jitter
            */            
            servo.write(servocenter);  
            Serial.println("center");  
          }else if (rssiDiff > 100 || (rssiLeft < 100 && rssiRight < 100) || rssiMidTop > rssiRight || rssiMidTop  > rssiLeft){
            /* 
            we seem to have lost sync, or have no lock.  at this point best to start auto seeking.#
            we pause every second to save battery, and allow rssi to stabilise.
            */
            Serial.println("seeking");  
            servo.write(servocenter +  speedFast); 
            delay(250);
            servo.write(servocenter);  
            delay(1000);
          } else {
                  /* 
                  simple filter to vary speed of servo when rssi difference is not too high
                  */
                    if(rssiDiff < 10){
                       if(rssiLeft > rssiRight) { 
                              servo.write(servocenter +  speedVSlow);   
                              Serial.println("move left very slow:");     
                       }else if(rssiRight > rssiLeft) { 
                              Serial.println("move right very slow:");  
                              servo.write(servocenter -  speedVSlow);
                       }                      
                   } else if(rssiDiff < 15){
                       if(rssiLeft > rssiRight) { 
                              servo.write(servocenter +  speedSlow);   
                              Serial.println("move left slow:");     
                       }else if(rssiRight > rssiLeft) { 
                              Serial.println("move right slow:");  
                              servo.write(servocenter -  speedSlow);
                       }                  
                   } else if(rssiDiff < 20){
                       if(rssiLeft > rssiRight) { 
                              servo.write(servocenter +  speedMedium);   
                              Serial.println("move left medium:");     
                       }else if(rssiRight > rssiLeft) { 
                              Serial.println("move right medium:");  
                              servo.write(servocenter - speedMedium);  
                       }               
                  } else {  
                       if(rssiLeft > rssiRight) { 
                              servo.write(servocenter +  speedFast);   
                              Serial.println("move left fast:");     
                       }else if(rssiRight > rssiLeft) { 
                              Serial.println("move right fast:");  
                              servo.write(servocenter - speedFast);  
                       } 
                  }      
     }
     
     //end
}
